#
# Try to find the FreeImage library and include path.
# Once done this will define
#
# FREEIMAGE_FOUND
# FREEIMAGE_INCLUDE_PATH
# FREEIMAGE_LIBRARY
# credits to http://code.google.com/p/nvidia-texture-tools/source/browse/trunk/cmake/FindFreeImage.cmake?r=580

IF (WIN32)
        FIND_PATH( FREEIMAGE_INCLUDE_PATH FreeImage.h
				PATH_SUFFIXES "Source"
				ENV PATH
                DOC "The directory where FreeImage.h resides")
        FIND_LIBRARY( FREEIMAGE_LIBRARY
                NAMES FreeImage freeimage
				ENV PATH
				PATH_SUFFIXES 
				"Dist"
				"Source"
				"Release"
				"Debug"
				"FreeImage"
                DOC "The FreeImage library")
ELSE (WIN32)
        FIND_PATH( FREEIMAGE_INCLUDE_PATH FreeImage.h
                /usr/include
                /usr/local/include
                /sw/include
                /opt/local/include
                DOC "The directory where FreeImage.h resides")
        FIND_LIBRARY( FREEIMAGE_LIBRARY
                NAMES FreeImage freeimage
                PATHS
                /usr/lib64
                /usr/lib
                /usr/local/lib64
                /usr/local/lib
                /sw/lib
                /opt/local/lib
                DOC "The FreeImage library")
ENDIF (WIN32)

SET(FREEIMAGE_LIBRARIES ${FREEIMAGE_LIBRARY})

IF (FREEIMAGE_INCLUDE_PATH AND FREEIMAGE_LIBRARY)
        SET( FREEIMAGE_FOUND TRUE CACHE BOOL "Set to TRUE if GLEW is found, FALSE otherwise")
ELSE (FREEIMAGE_INCLUDE_PATH AND FREEIMAGE_LIBRARY)
        SET( FREEIMAGE_FOUND FALSE CACHE BOOL "Set to TRUE if GLEW is found, FALSE otherwise")
ENDIF (FREEIMAGE_INCLUDE_PATH AND FREEIMAGE_LIBRARY)

MARK_AS_ADVANCED(
        FREEIMAGE_FOUND
        FREEIMAGE_LIBRARY
        FREEIMAGE_LIBRARIES
        FREEIMAGE_INCLUDE_PATH)

### additions of Dominik Rue ###
if(NOT FREEIMAGE_FOUND)
  message(FATAL_ERROR "FREEIMAGE not found")
endif(NOT FREEIMAGE_FOUND)

set ( MY_LIBS
  ${MY_LIBS}
  ${FREEIMAGE_LIBRARIES}
)

include_directories (
  ${FREEIMAGE_INCLUDE_PATH}
)

if (WIN32)
	## otherwise mingw32 won't link:
	#add_definitions("-DFREEIMAGE_LIB")
	#add_definitions("-DFREEIMAGE_EXPORTS")
	
	file(GLOB_RECURSE DLL_FILES "${FREEIMAGE_INCLUDE_PATH}/../*.dll")
    INSTALL(FILES ${DLL_FILES}
        DESTINATION bin
		COMPONENT main
    )
endif(WIN32)