#include <stdio.h>
#include <libexif/exif-loader.h>
#include <FreeImage.h>

#include <QDateTime>
#include <QString>
#include <QVector>
#include <QDebug>

extern "C" {
#include "jpeg-data.h"
}

bool changeTimeStamp(const QString& filenameIn,
                     const QString& filenameOut,
                     const QDateTime& newDate)
{
    ExifLoader *l;
    /* Create an ExifLoader object to manage the EXIF loading process */
    l = exif_loader_new();
    if (l) {
        ExifData *ed;

        /* Load the EXIF data from the image file */
        exif_loader_write_file(l, filenameIn.toLocal8Bit().data());

        /* Get a pointer to the EXIF data */
        ed = exif_loader_get_data(l);

        /* The loader is no longer needed--free it */
        exif_loader_unref(l);
        l = NULL;


        if (ed) {
            QVector<ExifTag> tags;
            tags.push_back(EXIF_TAG_DATE_TIME);
            tags.push_back(EXIF_TAG_DATE_TIME_ORIGINAL);
            tags.push_back(EXIF_TAG_DATE_TIME_DIGITIZED);

            for (int i=0; i<(int)tags.size(); i++) {
                /* See if this tag exists */
                ExifEntry *entry =exif_data_get_entry ( ed,
                                                        tags[i]);
                //exif_content_get_entry(ed->ifd[EXIF_IFD_0],
                //                                      EXIF_TAG_DATE_TIME);
                if (entry) {
                    if (entry->data) free (entry->data);
                    entry->components = 20;
                    entry->size = sizeof (char) * entry->components;
                    entry->data = (unsigned char*)malloc (entry->size);
                    if (!entry->data) {
                        throw "Not enough memory.";
                    }
                    strcpy ((char *) entry->data, newDate.toString("yyyy:MM:dd HH:mm:ss").toLocal8Bit().data());

                    /* Parse the JPEG file. */
                    JPEGData* jdata;
                    jdata = jpeg_data_new ();
                    jpeg_data_load_file (jdata, filenameIn.toLocal8Bit().data());

                    /* Make sure the EXIF data is not too big. */
                    unsigned char *d = NULL;
                    unsigned int ds;
                    exif_data_save_data (ed, &d, &ds);
                    if (ds) {
                        free (d);
                        if (ds > 0xffff)
                            throw "error correcting jpg time";
                    };

                    jpeg_data_set_exif_data (jdata, ed);

                    /* Save the modified image. */
                    jpeg_data_save_file (jdata, filenameOut.toLocal8Bit().data());
                    jpeg_data_unref (jdata);

                }
            }

            return true;
        }

    }
    return false;
}



int main(int argc, char** argv)
{

    if (argc != 3) {
        qDebug() << "usage: " << argv[0] << " imageIn imageOut";
        return -1;
    }

    if (changeTimeStamp(argv[1], argv[2], QDateTime::currentDateTime())) {
        qDebug() <<  "sucessful" ;
    } else {
         qDebug() <<  "failed"  ;
    }

}
