/***********************************************************************
 * This file is part of module_imageinfo.
 *
 * module_imageinfo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imageinfo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imageinfo.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QByteArray>
#include <QCryptographicHash>

#include "imagedata.h"
unsigned long ImageData::_lastId = 0;

ImageData::ImageData(const QString& filename)
  : _alreadyHasThumbnail(false)
  , _file(filename)
  , _isUsed(true)
  , _id(++_lastId)
  , _hashComputed(false)
  , _imageOrientation(MiscTools::OrientationJPG_normal)
  , _hasChanged(false)
{
}

QString ImageData::geHash() const
{
    if (!_hashComputed) {
        _computeHash();
    }
    return _hash;
}

void ImageData::_computeHash() const
{
    QFile file(_file.canonicalFilePath());

    if (file.open(QIODevice::ReadOnly)) {
        QByteArray fileData = file.readAll();

        QByteArray hashData = QCryptographicHash::hash(fileData,QCryptographicHash::Md5); // or QCryptographicHash::Sha1
        _hash = hashData.toHex();  // 0e0c2180dfd784dd84423b00af86e2fc
        _hashComputed = true;
    }
}

bool ImageData::hasChanged() const
{
    return _hasChanged;
}

void ImageData::setChangedStatus(const bool hasChanged)
{
    _hasChanged = hasChanged;
}

bool ImageData::hasThumbnail() const
{
    return _alreadyHasThumbnail;
}

unsigned long ImageData::getId() const
{
    return _id;
}

QFileInfo ImageData::getFileInfo() const
{
    return _file;
}

const MiscTools::pQImage &ImageData::getThumbnail() const
{
    return _thumbnail;
}

bool ImageData::getUsed() const
{
    return _isUsed;
}

void ImageData::setUsed(const bool isUsed)
{
    if (_isUsed == isUsed)
        return;
    _isUsed = isUsed;
    _hasChanged = true; // changed by user
}

void ImageData::setFileInfo(const QFileInfo& info)
{
    _file = info;
}

void ImageData::setHasThumbnail(const bool hasThumbnail)
{
    _alreadyHasThumbnail = hasThumbnail;
}

void ImageData::setThumbnail(const MiscTools::pQImage& image)
{
    _thumbnail = image;
}

MiscTools::OrientationJPG ImageData::getOrientation() const
{
    return _imageOrientation;
}

void ImageData::setOrientation(const MiscTools::OrientationJPG orientation)
{
    _imageOrientation = orientation;
}
