/***********************************************************************
 * This file is part of module_imageinfo.
 *
 * module_imageinfo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imageinfo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imageinfo.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef IMAGEDATA_H
#define IMAGEDATA_H

#include <memory>

#include <QFileInfo>
#include <QString>
#include <QStringList>

#include "orientationJPG.h"
#include "smartpointers.h"

class ImageData
{
 public:
  ImageData(const QString& filename="");

  virtual ~ImageData() {}

  bool hasThumbnail() const;
  unsigned long getId() const;
  QFileInfo getFileInfo() const;
  const MiscTools::pQImage &getThumbnail() const;
  bool getUsed() const;
  QString geHash() const;  
  MiscTools::OrientationJPG getOrientation() const;
  bool hasChanged() const;

  void setUsed(const bool isUsed);
  void setFileInfo(const QFileInfo& info);
  void setHasThumbnail(const bool hasThumbnail);
  void setThumbnail(const MiscTools::pQImage &image);
  void setOrientation(const MiscTools::OrientationJPG);
  void setChangedStatus(const bool hasChanged = false);

 protected:
  void _computeHash() const;

  bool _alreadyHasThumbnail;
  QFileInfo _file;
  MiscTools::pQImage _thumbnail;
  bool _isUsed;

  unsigned long _id;
  static unsigned long _lastId;

  mutable bool _hashComputed;
  mutable QString _hash;

  MiscTools::OrientationJPG _imageOrientation;

  bool _hasChanged;
};

typedef std::shared_ptr<ImageData> ImageDataPtr;

static QStringList _rawFormats = QStringList()
<< "3FR" //(Hasselblad)
<< "DCR"
<< "K25"
<< "KDC" //(Kodak)
<< "IIQ" //(Phase One)
<< "CR2" //(Canon)
<< "ERF" //(Epson)
<< "MEF" //(Mamiya),
<< "MOS" //(Leaf)
<< "NEF" //(Nikon),
<< "ORF" //(Olympus),
<< "PEF" //(Pentax),
<< "RW2" //(Panasonic) and
<< "ARW"
<< "SRF"
<< "SR2" //(Sony);
   ;


#endif // IMAGEDATA_H
