/***********************************************************************
 * This file is part of module_imagepreview.
 *
 * module_imagepreview is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imagepreview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imagepreview.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QCheckBox>
#include <QPainter>
#include <QDebug>

#include "imageviewdelegate.h"
#include "imageviewmodel.h"



ImageViewDelegate::ImageViewDelegate( QWidget *parent)
    : QItemDelegate(parent)
    , _parent(0)
{

}

QWidget * ImageViewDelegate::createEditor(QWidget *parent,
                                          const QStyleOptionViewItem &option,
                                          const QModelIndex &index) const
{
    return QItemDelegate::createEditor(parent, option, index);
}

void ImageViewDelegate::setEditorData(QWidget *editor,
                                      const QModelIndex &index) const
{
    return QItemDelegate::setEditorData(editor, index);
}

void ImageViewDelegate::setModelData(QWidget *editor,
                                     QAbstractItemModel *model,
                                     const QModelIndex &index) const
{
    return QItemDelegate::setModelData(editor, model, index);
}

void ImageViewDelegate::updateEditorGeometry(QWidget *editor,
                                             const QStyleOptionViewItem &option,
                                             const QModelIndex& ) const
{
    editor->setGeometry(option.rect);
}

void ImageViewDelegate::paint ( QPainter * painter,
                               const QStyleOptionViewItem & option,
                               const QModelIndex & index ) const
{

    if (index.column() == IMAGE_VIEW_PREVIEW_IDX) {
        QIcon im;

        QStyleOptionViewItem myOption = option;
        myOption.displayAlignment = Qt::AlignCenter;
        QRect rectIm = myOption.rect;
        if (index.model()->data(index, Qt::DecorationRole).canConvert<QIcon>()) {
            im = index.model()->data(index, Qt::DecorationRole).value<QIcon>();

            QSize s = im.pixmap(option.rect.width(), option.rect.height()).size();

            rectIm.setY(rectIm.y() + (rectIm.height() - s.height())/2);
            rectIm.setX(rectIm.x() + (rectIm.width() - s.width())/2);
            rectIm.setWidth(s.width());
            rectIm.setHeight(s.height());
        }

        painter->drawPixmap(rectIm, im.pixmap(rectIm.size()));
    } else {
        return QItemDelegate::paint(painter, option, index);
    }
}


QSize ImageViewDelegate::sizeHint ( const QStyleOptionViewItem & option,
                                   const QModelIndex & index ) const
{    
    if (index.column() == IMAGE_VIEW_PREVIEW_IDX
            && index.model()->data(index, Qt::DecorationRole).canConvert<QIcon>()) {
        QIcon im;
        im = index.model()->data(index, Qt::DecorationRole).value<QIcon>();
        QSize s = im.actualSize(QSize(_thumbnailsize, _thumbnailsize));
        if (s.width() > s.height()) {
            return QSize(_thumbnailsize, _thumbnailsize*s.height()/s.width());
        } else {
            return QSize(_thumbnailsize*s.width()/s.height(), _thumbnailsize);
        }
    } else {
        QSize sizehint = QItemDelegate::sizeHint(option, index);
        //sizehint.setHeight( qMax(sizehint.height(), _thumbnailsize));
        sizehint.setHeight( sizehint.height());
        return sizehint;
    }
}
