/***********************************************************************
 * This file is part of module_imagepreview.
 *
 * module_imagepreview is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imagepreview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imagepreview.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef QIMAGEVIEWMODEL_H
#define QIMAGEVIEWMODEL_H

#include <QAbstractTableModel>

#include "imagedata.h"
#include "smartpointers.h"

#define IMAGE_VIEW_PREVIEW_IDX 0
#define IMAGE_VIEW_FILENAME_IDX 1
#define IMAGE_VIEW_COLUMNS 2

class ImageViewModel : public QAbstractTableModel
{
    Q_OBJECT

public:
    explicit ImageViewModel(QObject *parent = 0);

    virtual ~ImageViewModel() {}

    virtual int rowCount(const QModelIndex& parent) const;
    virtual int columnCount(const QModelIndex& parent) const;

    virtual QVariant data(const QModelIndex& index, int role) const;
    virtual QVariant data(const int row, const int col, int role) const;

    virtual QVariant headerData(int section,
                        Qt::Orientation orientation,
                        int role) const;

    virtual bool setData ( const QModelIndex & index,
                           const QVariant & value,
                           int role = Qt::EditRole );
    virtual Qt::ItemFlags flags ( const QModelIndex & index ) const;

    const QList<ImageDataPtr>& getList() const;
    virtual void setList(const QList<ImageDataPtr>&);
    virtual void updateList(const int pos, const ImageData&);

    static MiscTools::pQImage noThumb;
    static MiscTools::pQImage errorThumb;


signals:

public slots:
    void selectAll();
    void deselectAll();
    void invertSelection();

protected:
    static MiscTools::pQImage _initNoThumb();
    static MiscTools::pQImage _initErrThumb();

    void _updateViews(const int row = -1);

    QList<ImageDataPtr> _images;

    mutable int _currentEditing;
};


#endif // QIMAGEVIEWMODEL_H
