/***********************************************************************
 * This file is part of module_imageviewer.
 *
 * module_imageviewer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imageviewer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imageviewer.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include "imageviewerloader.h"

#include <QDebug>
#include <QSvgRenderer>
#include <QDesktopWidget>
#include <QPainter>

#include "misctools.h"


ImageViewerLoader::ImageViewerLoader(const QString filename,
                                     QObject *parent)
    : QThread(parent)
    , _file (filename)
    , _imSize (MiscTools::MiscTools::retrieveDesktopResolution().size())
{
    //connect(this, SIGNAL(finished()), SLOT(deleteLater()));
}

ImageViewerLoader::~ImageViewerLoader()
{
    wait();
}

void ImageViewerLoader::deleteInstance()
{
    // is it necessary to put this at the end of the even
    this->deleteLater();
}


void ImageViewerLoader::run()
{
    MiscTools::pQImage image;
    try {
        image = MiscTools::MiscTools::loadImageFreeImage(_file);

        if (image->isNull() || image->width() == 0 || image->height() == 0) {
            // now try to load as SVG
            QSvgRenderer renderer(_file);
            if (renderer.isValid()) {
                //const QRect deskSize = MiscTools::MiscTools::retrieveDesktopResolution();
                const double deskRatio = _imSize.width() / double(_imSize.height());
                const QSize suggestedSize = renderer.defaultSize();
                const double suggRatio = suggestedSize.width() / double(suggestedSize .height());
                QSize targetSize;
                if (deskRatio <= suggRatio) {
                    targetSize.setWidth(_imSize.width());
                    targetSize.setHeight(_imSize.width()/suggRatio);
                } else {
                    targetSize.setHeight(_imSize.height());
                    targetSize.setWidth(_imSize.height()*suggRatio);
                }
                //qDebug() << _file << deskRatio << suggRatio <<targetSize;
                *image = QImage(targetSize, QImage::Format_ARGB32);
                image->fill(Qt::white);

                QPainter painter(image.get());
                //painter.setBackground(Qt::green);
                //painter.setBrush(Qt::red);
                //painter.setBackgroundMode(Qt::OpaqueMode);
                //painter.setPen(Qt::white);*/
                renderer.render(&painter);
                painter.end();
                *image = image->convertToFormat(QImage::Format_RGB32, Qt::ImageConversionFlag::DiffuseDither | Qt::ImageConversionFlag::ThresholdAlphaDither);

            }
            if (image->isNull() || image->width() == 0 || image->height() == 0) {
                // try to load as QImage now
                image->load(_file);
                if (!(image->isNull() || image->width() == 0 || image->height() == 0)) {
                    *image = image->convertToFormat(QImage::Format_RGB32, Qt::ImageConversionFlag::DiffuseDither | Qt::ImageConversionFlag::ThresholdAlphaDither);
                    *image = image->scaled(MiscTools::MiscTools::retrieveDesktopResolution().size(),Qt::KeepAspectRatio, Qt::SmoothTransformation);
                }
            }
        } else {
            *image = image->scaled(_imSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        }

        emit doneLoading(image, _file);
    } catch (...) {
        qDebug() << "error encountered on loading thread";
    }
}
