/***********************************************************************
 * This file is part of module_misc.
 *
 * module_misc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_misc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_misc.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef MISC_TOOLS_H
#define MISC_TOOLS_H

#include <QString>
#include <QImage>
#include <QTranslator>
#include <QStringList>
#include <QVariant>
#include <QFileInfo>

#include <FreeImage.h>

#include "orientationJPG.h"

#include "smartpointers.h"


namespace MiscTools
{

#define TOOLS_TIME_FORMAT "yyyy:MM:dd HH:mm:ss"

class MiscTools
{
public:
#ifdef _WIN32
	static QString getDevicePath(const QString& deviceID,
								 const QString& number);
#endif
    static QString appDataPath();

    static pQImage loadImageFreeImage(const QString& filename);

    static QByteArray getMD5Sum(const QString & path);

    static bool getTimeStampAndOrientation(const QString& filename,
                                           QDateTime& timeOut,
                                           OrientationJPG& orientationOut);

    static bool changeTimeStamp(const QString& filenameIn,
                                const QString& filenameOut,
                                const QDateTime& newDate,
                                const bool saveInTextIfNotWOrks);

    static bool lockDialog();
    static void unlockDialog();

    static QRect retrieveDesktopResolution();

private:
    MiscTools() {}

    static bool _dialogLock;
};

template <typename T>
void setAllToolTips(T* w,
                    const QString& toolTip,
                    const QString& whatsThis = "",
                    const QString& statusTip  = "")
{
    w->setToolTip(toolTip);
    if (whatsThis.length() == 0) {
        w->setWhatsThis(toolTip);
    } else {
        w->setWhatsThis(whatsThis);
    }

    if (statusTip.length() == 0) {
        w->setStatusTip(toolTip);
    } else {
        w->setStatusTip(statusTip);
    }
}

} // namespace MiscTools

#endif // MISC_TOOLS_H
