/***********************************************************************
 * This file is part of module_progressdialog.
 *
 * module_progressdialog is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_progressdialog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_progressdialog.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QFontMetrics>
#include <QDebug>

#include "progressdialog.h"
#include "ui_progressdialog.h"

ProgressDialog::ProgressDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ProgressDialog),
    _isCancelled(false)
{
    ui->setupUi(this);
    _timer.start();
}

void ProgressDialog::closeEvent ( QCloseEvent * )
{
    emit canceled();
}

ProgressDialog::~ProgressDialog()
{
    delete ui;
}

void ProgressDialog::setMinMax(const int min, const int max)
{
    ui->progressBar->setMinimum(min);
    ui->progressBar->setMaximum(max);
}

void ProgressDialog::setLabel(const QString& str)
{
    QFontMetrics fm( ui->label_whatsOn->font() );
    ui->label_whatsOn->setText("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW");
    const int w = ui->label_whatsOn->width();
    if( w < fm.width( str ) ){
        ui->label_whatsOn->setToolTip( str );
        ui->label_whatsOn->setText( fm.elidedText( str, Qt::ElideMiddle, w ));
    } else {
        ui->label_whatsOn->setToolTip( str );
        ui->label_whatsOn->setText( str );
    }

    _timer.restart();
    qApp->processEvents();
}

void ProgressDialog::setValue(const int value)
{
    ui->progressBar->setValue(value);
    _timer.restart();
    qApp->processEvents();
}

void ProgressDialog::on_buttonBox__rejected()
{
    _isCancelled = true;
    emit canceled();
}

bool ProgressDialog::isCanceled()
{
    return _isCancelled;
}

void ProgressDialog::resetCancel()
{
    _isCancelled = false;
}

void ProgressDialog::setTitle(const QString& title)
{
    QDialog::setWindowTitle(title);
}

bool ProgressDialog::checkUpdateWanted() const
{
    if (_timer.elapsed() >= UPDATE_EVERY_MS) {
        return true;
    } else {
        return false;
    }
}
