/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QDateTime>
#include <QIcon>
#include <QDebug>
#include <QPen>
#include <QPainter>

#include "adapttimesdelegate.h"

AdaptTimesDelegate::AdaptTimesDelegate(QObject *parent)
    : QItemDelegate(parent)
    , _anchorPos(-1)
    , _targetPos(-1)
    , _anchor(QPixmap::fromImage(QImage(":/images/anchor_image.png")))
    , _clock(QPixmap::fromImage(QImage(":/images/clock_full.png")))
{
}

void AdaptTimesDelegate::paint(QPainter *painter,
                               const QStyleOptionViewItem &option,
                               const QModelIndex &index) const
{
    QString val(index.model()->data(index, Qt::DisplayRole).toString());
    QIcon im;

    QStyleOptionViewItem myOption = option;
    myOption.displayAlignment = (index.column() == 0 ? Qt::AlignRight : Qt::AlignLeft) | Qt::AlignVCenter;
    QRect rectText = myOption.rect;
    QRect rectIm = myOption.rect;
    bool drawFont = true;
    if (index.model()->data(index, Qt::DecorationRole).canConvert<QIcon>()) {
        im = index.model()->data(index, Qt::DecorationRole).value<QIcon>();

        QSize s = im.pixmap(10000,option.rect.height()).size();

        if (s.width() > rectIm.width()) {
            // aspekt  ratio:
            const float ratio = (float)s.width() / (float)s.height();
            rectIm.setHeight( (float)rectIm.width() / ratio);
            drawFont = false;
        } else {
            if (index.column() == 0) {
                rectIm.setX(rectIm.x() + rectIm.width() - s.width());
                rectText.setWidth(rectText.width() - rectIm.width());
            } else {
                rectIm.setWidth(s.width());
                rectText.setX(rectText.x() + rectIm.width());
            }
        }
    }

    if (drawFont) {
        painter->drawText(rectText, myOption.displayAlignment | Qt::TextWordWrap, val );
    }

    if ( (index.row() == _anchorPos && index.column() == 0)
            ||
            (index.row() == _targetPos && index.column() == 1)
         ) {
        if (index.column() == 0) {
            MiscTools::pQPixmap anchorRes = im.pixmap(rectIm.size());
            MiscTools::pQPixmap anchorScaled = ( _anchor->height() < anchorRes->height()
                                     && _anchor->width() < anchorRes->width()) ?
                        _anchor
                      : _anchor->scaled(rectIm.size(),
                                       Qt::KeepAspectRatio,
                                       Qt::SmoothTransformation);
            QPainter painterAnchor(anchorRes.get());
            const int w = rectIm.width() - anchorScaled->width(),
                    h = rectIm.height() - anchorScaled->height();
            painterAnchor.drawPixmap(QRectF(QPointF(w, h),
                                            anchorScaled->size()),
                                     *anchorScaled,
                                     QRectF(QPointF(0,0),
                                            anchorScaled->size())
                                     );
            painter->drawPixmap(rectIm, *anchorRes);
        } else {
            MiscTools::pQPixmap clockRes (im.pixmap(rectIm.size()));
            MiscTools::pQPixmap clockScaled ( _clock->height() < clockRes->height()
                                     && _clock->width() < clockRes->width() ?
                        _clock
                      : _clock->scaled(rectIm.size(),
                                       Qt::KeepAspectRatio,
                                       Qt::SmoothTransformation) );
            QPainter painterClock(clockRes.get());
            const int w = rectIm.width() - clockScaled->width(),
                    h = rectIm.height() - clockScaled->height();
            painterClock.drawPixmap(QRectF(QPointF(w, h),
                                            clockScaled->size()),
                                     *clockScaled,
                                     QRectF(QPointF(0,0),
                                            clockScaled->size())
                                     );
            painter->drawPixmap(rectIm, *clockRes);
        }

        QPen pen, penOld = painter->pen();
        pen.setStyle(Qt::SolidLine);
        pen.setWidth(3);
        pen.setBrush(Qt::green);
        painter->setPen(pen);

        rectIm.setWidth(rectIm.width() - 1);
        rectIm.setHeight(rectIm.height() - 1);
        painter->drawRect(rectIm);
        painter->setPen(penOld);
    } else {
        painter->drawPixmap(rectIm, im.pixmap(rectIm.size()));
    }
}

void AdaptTimesDelegate::setThumbnailSize(const int size)
{
    _thumbnailSize = size;
}

QSize AdaptTimesDelegate::sizeHint ( const QStyleOptionViewItem &, const QModelIndex & index ) const
{
    if (index.model()->data(index, Qt::DecorationRole).canConvert<QIcon>()) {
        QIcon im;
        im = index.model()->data(index, Qt::DecorationRole).value<QIcon>();
        QSize s = im.actualSize(QSize(_thumbnailSize, _thumbnailSize));
        if (s.width() > s.height()) {
            return QSize(_thumbnailSize, _thumbnailSize*s.height()/s.width());
        } else {
            return QSize(_thumbnailSize*s.width()/s.height(), _thumbnailSize);
        }

    } else {
        return QSize(_thumbnailSize, _thumbnailSize);
    }
}


void AdaptTimesDelegate::setAnchorPos(const int pos)
{
    _anchorPos = pos;
}

void AdaptTimesDelegate::setTargetPos(const int pos)
{
    _targetPos = pos;
}
