/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef ADAPTTIMESDIALOG_H
#define ADAPTTIMESDIALOG_H

#include <QDialog>
#include <QMatrix>
#include <QTableWidgetItem>
#include "adapttimesdelegate.h"
#include <imagedataextended.h>

namespace Ui {
    class AdaptTimesDialog;
}

class TimeDiff
{
    int dirOne;
    int dirTwo;
    unsigned long seconds;
};

struct SortHelp
{
    ImageDataPtr imagedata;
    QString subDir;
};

class AdaptTimesDialog : public QDialog
{
    Q_OBJECT

public:
    explicit AdaptTimesDialog(QWidget *parent = 0);
    ~AdaptTimesDialog();

    void setDirectories(const QStringList& directories, const QList<ImageDataPtr> &fileList, const int baseLength);

    void loadSettings();

    void clear();

signals:
    void doneEditing();

public slots:

    void newThumbnail(const QString& filepath);

    void showInfo();

    void _on_pushButton_finish_clicked();

    void display();

private slots:

    void on_pushButton_reset_clicked();

    void _displayList();

    void _recomputeInputDiff();

    void on_spinBox_thumb_editingFinished();

    void on_tableWidget_comparison_itemSelectionChanged();

    void on_spinBox_thumb_valueChanged(int arg1);

    void on_toolButton_setAnchor_clicked();

    void on_toolButton_justBefore_clicked();


    void on_toolButton_justAfter_clicked();

    void _contextMenuEvent(const QPoint &pos);

    void _onViewAll(const int row=-1);
    void _onViewReference(const int row=-1);
    void _onViewTarget(const int row=-1);

private:

    void _setTableWidget(const ImageDataExtended* tmp,
                         QTableWidgetItem** date,
                         QTableWidgetItem** newItem,
                         const int posIm);

    void _updateKeepAnchors();
    void _updateImagesTimes(const int before);

    bool _resetTimes();

    void _computeTimeDifference();

    void _updateFields();

    void _updateAnchorsAndTimes(const int before);

    void closeEvent(QCloseEvent *);

    Ui::AdaptTimesDialog *ui;

    AdaptTimesDelegate _adaptTimesDelegate;

    QList<ImageDataPtr> _images;
    int _baseLength;

    QStringList _directories;


    QAction *_viewTarget ;
    QAction *_viewReference;
    QAction *_viewAll ;


    bool _init;

    int _anchorPos;
    int _targetPos;
    QVector<QVector<int> > _timeDiff;

    QList<SortHelp> _currentList;

    bool _infoHasBeenShown;
};

#endif // ADAPTTIMESDIALOG_H

