/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QDateTime>
#include <QDateTimeEdit>

#include "albumdelegate.h"

AlbumDelegate::AlbumDelegate(QObject *parent) :
    QItemDelegate(parent)
{
}

void AlbumDelegate::paint(QPainter *painter,
                          const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
{
    if (index.column() == 1 || index.column() == 2) {
        QDateTime val(QDateTime::fromTime_t(index.model()->data(index, Qt::DisplayRole).toUInt()));
        QStyleOptionViewItem myOption = option;
        myOption.displayAlignment = Qt::AlignRight | Qt::AlignVCenter;
        drawDisplay(painter, myOption, myOption.rect, val.toString(Qt::DefaultLocaleLongDate));
        drawFocus(painter, myOption, myOption.rect);
    } else {
        QItemDelegate::paint(painter, option, index);
    }
}

QWidget * AlbumDelegate::createEditor(QWidget *parent,
                                      const QStyleOptionViewItem &option,
                                      const QModelIndex &index) const
{
    if (index.column() == 1 || index.column() == 2) {
        QDateTimeEdit *dateEdit = new QDateTimeEdit(parent);
        //dateEdit->setDisplayFormat(Qt::DefaultLocaleShortDate);
        dateEdit->setCalendarPopup(true);
        connect(dateEdit, SIGNAL(editingFinished()),
                this, SLOT(commitAndCloseEditor()));
        return dateEdit;
    } else {
        return QItemDelegate::createEditor(parent, option, index);
    }
}


void AlbumDelegate::commitAndCloseEditor()
{
    QDateTimeEdit *editor = qobject_cast<QDateTimeEdit*>(sender());
    emit commitData(editor);
    emit closeEditor(editor);
}

void AlbumDelegate::setEditorData(QWidget *editor,
                                  const QModelIndex &index) const
{
    if (index.column() == 1 || index.column() == 2) {
        uint value = index.model()->data(index, Qt::DisplayRole).toUInt();
        QDateTimeEdit * dateEdit =  qobject_cast<QDateTimeEdit*>(editor);
        dateEdit->setDateTime(QDateTime::fromTime_t(value));
    } else {
        QItemDelegate::setEditorData(editor, index);
    }
}

void AlbumDelegate::setModelData(QWidget *editor,
                                 QAbstractItemModel *model,
                                 const QModelIndex &index) const
{
    if (index.column() == 1 || index.column() == 2) {
        QDateTimeEdit * dateEdit =  qobject_cast<QDateTimeEdit*>(editor);
        model->setData(index, dateEdit->dateTime().toTime_t());
    } else {
        QItemDelegate::setModelData(editor, model, index);
    }
}
