/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef COPYFILES_H
#define COPYFILES_H

#include <QString>
#include <QObject>
#include <QVector>
#include <QStringList>
#include <QList>
#include <QPair>

#include "imagedataextended.h"
#include "eventselectionassistant.h"
#include "mainsettings.h"

#include "copyfiles_helper.h"

#define DATABASE_MD5_TABLE "imagescopied"

class CopyFiles : public QObject
{
    Q_OBJECT

public:

    static QString getTag(const QString& key);
    static QString getTagDesc(const QString& key);

    static QVector<PictureTag> tags;

    static CopyItem applyTags(
            const QStringList& destinationDescriptions,
            const QString & currentDir,
            const QVector<PhotoAlbum>& albums,
            const QList<QPair<QString, QString> >& subDirs,
            const QMap<int, QDateTime> earliestPerAlbum,
            const QMap<int, QDateTime> latestPerAlbum,
            const bool albumHasNoDayTime,
            const QString& albumKnownDayTags,
            const QString& knownDayTag,
            const QString& albumUnknownDayTags,
            const QString& unknownDayTag,
            ImageDataExtended *image,
            QList<ImageDataExtended *> &noAlbumError,
            QList<ImageDataExtended *> &noTimeError);

    static QVector<CopyItem> applyForAllFiles(
            const QStringList& descriptions,
            const QString & currentDir,
            const QVector<PhotoAlbum>& albums,
            const QList<QPair<QString, QString> >& subDirs,
            const QList<ImageDataPtr> images,
            const bool jpgAndRAW,
            const bool unique,
            const bool copyOnlyOnce,
            const QSqlDatabase* db,
            const QString& albumKnownDayTags,
            const QString& knownDayTag,
            const QString& albumUnknownDayTags,
            const QString& unknownDayTag,
            const bool forceSameNumber,
            const HandleExistingItems existingFiles,
            QList<QPair<QString, QString> >& moveExistingFiles,
            QList<ImageDataExtended*>& noAlbumError,
            QList<ImageDataExtended*>& noTimeError,
            QList<ImageDataExtended*>& removeUnique,
            QList<ImageDataExtended*>& alreadyCopied,
            QList<ImageDataExtended*>& notSelected);

    static bool doCopy(
            const QStringList& descriptions,
            const QString & currentDir,
            const QVector<PhotoAlbum>& albums,
            const QList<QPair<QString, QString> >& subDirs,
            const QList<ImageDataPtr> imagesOrig,
            const bool jpgAndRAW,
            const bool unique,
            const bool updateMetadata,
            const bool saveTextFileIfFailure,
            const bool copyOnlyOnce,
            const bool createOwnerShipTextFile,
            const QSqlDatabase* db,
            const QString& albumKnownDayTags,
            const QString& knownDayTag,
            const QString& albumUnknownDayTags,
            const QString& unknownDayTag,
            const CopyOverwriteMode overwriteMode,
            const bool forceSameNumbers,
            const HandleExistingItems existingFiles,
            QList<QPair<QString, QString> > &moveExistingFiles,
            QWidget *parent,
            QList<ImageDataExtended*>& noAlbumError,
            QList<ImageDataExtended*>& noTimeError,
            QList<ImageDataExtended*>& removeUnique,
            QList<ImageDataExtended *> &alreadyCopied, QList<ImageDataExtended *> &notSelected, QList<ImageDataExtended *> &copyError);

    static bool checkHasBeenCopied(
            const ImageDataExtended* data,
            const QSqlDatabase* db);

    static void initializeTags();

private:

    static int _getNewRunnNumber(const CopyItem& currCopyItem,
                                 const QVector<CopyItem>& out,
                                 const bool forceSameNumber,
                                 const QVector<QPair<QString, QString> > &tagsAndDateMap,
                                 const HandleExistingItems existingFiles,
                                 const int k,
                                 QStringList &filenamesExisting,
                                 QVector<DestinationInfo>& destinationRunningNumbers);

    static void _updateMoveExisting(const QString &newFilename,
                                    const QVector<QPair<int, int> > &startAndLengthOfRunnNumbers,
                                    const int desiredNumber,
                                    const int numberOfDigits,
                                    const QString &path,
                                    QList<QPair<QString, QString> > &moveExistingFiles);

};

#endif // COPYFILES_H
