/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef COPYFILES_HELPER_H
#define COPYFILES_HELPER_H

#include <QString>
class QDateTime;

struct PictureTag
{
    QString tag;
    QString description;
    QString key;
    QString QDateFormatEquivalent;

    PictureTag() {}
    PictureTag(const QString& t,
               const QString& d,
               const QString& k,
               const QString& f="")
        : tag(t)
        , description(d)
        , key(k)
        , QDateFormatEquivalent(f)
    {}
};

struct CopyItem
{
    ImageDataExtended* image;
    /**
      * first is destination filename
      * second is destination directory
      */
    QList<QPair<QString, QString> > destinations;
    QVector<int> serialNumbers;
    QVector<QString> destinationsNoDatesFilled;
    QString subdir;
};

enum HandleExistingItems
{
    HandleExistingItems_PerformAction,
    HandleExistingItems_Merge,
    HandleExistingItems_Continue,
    HandleExistingItems_ContinueAndRenameExisting
};

struct DestinationInfo
{
    QString path;
    int runningNumber;
    QStringList existingFilesWithSamePattern;
    QVector<int> existingRunningNumbers;
    QVector<QVector<QPair<int, int> > > startAndLengthOfRunnNumbers;
    QString filepattern;

    DestinationInfo()
        : runningNumber (0)
    {}

    DestinationInfo(const QString& path,
                    const int runningNumber,
                    const QStringList& existingFilesWithSamePattern,
                    const QVector<int>& existingRunningNumbers,
                    const QVector<QVector<QPair<int, int> > >& startAndLengthOfRunnNumbers,
                    const QString& filepattern)
        : path(path)
        , runningNumber(runningNumber)
        , existingFilesWithSamePattern(existingFilesWithSamePattern)
        , existingRunningNumbers(existingRunningNumbers)
        , startAndLengthOfRunnNumbers(startAndLengthOfRunnNumbers)
        , filepattern(filepattern)
    {}
};

#endif // COPYFILES_HELPER_H
