/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2013, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef EVENTSELECTIONASSISTANT_H
#define EVENTSELECTIONASSISTANT_H

#include <QDialog>
#include <QDateTime>
#include <QVector>
#include <QVariant>

#include <imagedataextended.h>
#include <thumbnailloader.h>
#include "imagewidget.h"

namespace Ui {
    class EventSelectionAssistant;
}

struct PhotoAlbum
{
    QString name;
    QDateTime from;
    QDateTime to;
    int ID;
};

QDataStream &operator<<(QDataStream &out, const PhotoAlbum &album);
QDataStream &operator>>(QDataStream &in, PhotoAlbum& album);

struct UndoItem
{
    QDateTime currDateFrom;
    QDateTime currDateTo;
    QVector<PhotoAlbum> albums;
};

Q_DECLARE_METATYPE(PhotoAlbum)
Q_DECLARE_METATYPE(QVector<PhotoAlbum>)

#define ALBUMS_FILE_NUMBER 0xA0B0C0D1
#define ALBUMS_DATA_FILE "albums.qt.data"

class EventSelectionAssistant : public QDialog
{
    Q_OBJECT

public:
    explicit EventSelectionAssistant(QWidget *parent = 0);
    ~EventSelectionAssistant();

    void setImageList(const QList<ImageDataPtr>& photoList,
                      const int baseDirLength);

    QVector<PhotoAlbum> getAlbums();

    void loadSettings();
    void resetLimitOutside();

signals:
    void tableViewSelected(const bool enabled);

    void finishedAlbums();

public slots:
    void loadAlbumsFromFile(const QString& location);
    void resetData(const QString& location);

    bool eventFilter(QObject *target, QEvent* event);

    void showInfo();

    void _on_pushButton_finish_clicked();


private slots:
    void on_checkBox_showFixed_toggled(bool checked);
    void _selectSizes(const int thumbsize);

    void on_pushButton_addAlbum_clicked();

    void onLoadedThumbnails(const QFileInfo, const MiscTools::pQImage);
    void onErrorThumbnails(const QFileInfo);

    void on_spinBox_thumbnailSize_editingFinished();

    void on_spinBox_thumbnailSize_valueChanged(int arg1);

    void on_checkBox_random_toggled(bool checked);

    void on_toolButton_reselect_clicked();

    void on_checkBox_limitOutside_toggled(bool limit);

    void on_spinBox_numOutside_valueChanged(int arg1);
    void on_toolButton_deleteRow_clicked();

    void on_toolButton_nextImages_clicked();

    void on_toolButton_showImages_clicked();

    void on_toolButton_updateCurrImages_clicked();

    void onTimeFromForNewTriggered(const QDateTime&);
    void onTimeToForNewTriggered(const QDateTime&);
    void onTimeFromForTableTriggered(const QDateTime&);
    void onTimeToForTableTriggered(const QDateTime&);

    void displayDate(const bool onlyNotYetCovered = false );

    void updateUndoList();

    void on_toolButton_undo_clicked();

    void on_toolButton_redo_clicked();

    void itemSelectionChanged();

    void _viewAlbums(const QList<ImageDataPtr>& imageList,
                     const QString& path);
    void _viewAllImages(const QString& path);
    void _viewCurrentAlbum(const QString& path);

    void on_toolButton_clicked();


private:


    void _setNextUncoveredTime();

    bool _getNextUncoveredDate(QDateTime& newDate);

    void _clearImages();

    void _checkImageWidget(const ImageDataExtended& search);

    void _setAlbums(const QVector<PhotoAlbum>& albums);

    void _connectImageWidget(ImageWidget* w);

    int _getSelectedRow(const bool warning = true);

    void _setCurrentUndo();

    void _writeAlbumsToFile();

    Ui::EventSelectionAssistant *ui;

    QList<ImageDataPtr> _photoList;

    QList<ImageWidget*> _widgetsInside;
    QList<ImageWidget*> _widgetsBefore;
    QList<ImageWidget*> _widgetsAfter;

    QDateTime _currFrom;
    QDateTime _currTo;
    QDateTime _lastSkippedTo;

    bool _init;

    bool _dontUndo;

    QList<UndoItem> _undoHistory;
    int _undoPosition;

    QString _settingsLocation;

    int _baseDirLength;

    static qint64 _lastID;

    bool _infoHasBeenShown;

    unsigned int _currLimit;
};

#endif // EVENTSELECTIONASSISTANT_H
